# author: Natapol Pornputtapong <natapol.por@gmail.com>
#
# Documentation: Natapol Pornputtapong (RDoc'd and embellished by William Webber)
#
require 'json'

module Sylfy
    # Model template
    class DataModel
        
        class DuplicateError < Exception
            
        end
        
        attr_reader :data
        # Initialize function
        # @param data [Hash] data object
        def initialize(data = nil)
            @data = {class: self.class.to_sym}
            if data
                data.each_pair do |field, value|
                    self.send "#{field}=".to_sym, value if self.method_defined?(field)
                end
            end
            
        end
        
        # convert data object to JSON
        # @return [JSON]
        def to_json()
            return @data.to_json
        end
        # @return [String]
        def to_str()
            return JSON::pretty_generate(@data)
        end
        # @return [Hash]
        def to_hash()
            return @data
        end
        # Test if data object is valid
        def valid?()
            if @@validates
                result = true
                @@validates.each_pair do |field, rules|
                    rules.each_pair do |type, rule|
                        case type.to_sym
                        when :type
                            if rule.kind_of?(Array)
                                rule.each {|e| result &&= self.assert_type_of?(e, @data[field.to_sym])}
                            else
                                result &&= self.assert_type_of?(rule, @data[field.to_sym])
                            end
                        when :presence
                            result &&= !(rule ^ (@data[field.to_sym] != nil && !@data[field.to_sym].empty?))
                        when :format
                            if rule.kind_of?(Array)
                                rule.each {|e| result &&= self.assert_match?(e, @data[field.to_sym])}
                            else
                                result &&= self.assert_match?(rule, @data[field.to_sym])
                            end
                        end
                    end
                end
                return result
            else
                return true
            end
        end
        
        def assert_type_of?(crass, value)
            if crass < DataModel
                return crass.new(value).valid?
            else
                return value.kind_of?(crass)
            end
        end
        
        protected :assert_type_of?
        
        def assert_match?(pattern, value)
            return value.kind_of?(String) && value =~ pattern ? true : false
        end
        
        protected :assert_match?
        
        def self.has_one(*attrs)
            @@has_one ||= []
            attrs.each do |attr|
                if !self.method_defined?(attr)
                    @@has_one.push(attr)
                    self.class_eval do
                        define_method(attr) do
                            @data[attr]
                        end
                        
                        define_method("#{attr}=") do |param|
                            @data[attr] = param.kind_of?(DataModel) ? param.to_hash : param
                        end
                    end
                else
                    raise DuplicateError, "#{attr.inspect} is already defined."
                end
            end
        end
        
        def self.has_many(*attrs)
            @@has_many ||= []
            attrs.each do |attr|
                if !self.method_defined?(attr)
                    @@has_many.push(attr)
                    self.class_eval do
                        define_method(attr) do
                            @data[attr] = []
                        end
                        
                        define_method("#{attr}=") do |*param|
                            @data[attr] = param.flatten
                        end
                        
                        define_method("#{attr}_add") do |*param|
                            @data[attr] += param.flatten
                        end
                    end
                else
                    raise DuplicateError, "#{attr.inspect} is already defined."
                end
            end
        end
        
        def self.validates(*attrs, rules)
            @@validates ||= {}
            attrs.each do |attr|
                @@validates[attr] ||= {}
                @@validates[attr].merge!(rules)
            end
        end
        
        #class << self;self;end.send :protected, :subjects_xml, :subjects_html
    end
end
