# 
# UniSysDB library in Ruby
# Copyright (C) 2012
#
# author: Natapol Pornputtapong <natapol@chalmers.se>
#
# Documentation: Natapol Pornputtapong (RDoc'd and embellished by William Webber)
#

require 'gsl'

module Sylfy
    # additional Mathematic function
    module Mathf
        
        # calculate log factorial
        # @param n [Int]
        def self.lnfactorial(n)
            return GSL::Sf::lngamma(n+1)
        end
        
        # calculate log combinatorial
        # @param n [Int] the population size
        # @param r [Int] the number of draws
        def self.lncombinatorial(n, r)
            return lnfactorial(n) - lnfactorial(r) - lnfactorial(n-r)
        end
        
        # hypergeometric function
        # @param nn [Int] the population size
        # @param kk [Int] the number of success states in the population
        # @param n [Int] the number of draws
        # @param k [Int] the number of successes
        def self.hypergeometric(k, nn, kk, n)
            log_h = lncombinatorial(kk, k) + lncombinatorial(nn-kk, n-k) - lncombinatorial(nn, n)
            return (Math::E ** log_h)
        end
        
    end
end
