# 
# UniSysDB library in Ruby
# Copyright (C) 2012
#
# @author Natapol Pornputtapong <natapol@chalmers.se>
#
require 'open-uri'
require 'net/http'
require 'json'

module Sylfy
	
	module Service
		
		
		module CIR
            
            module REST
				@@baseuri = "http://cactus.nci.nih.gov/chemical/structure"
				@@representation_type = [:stdinchi, :stdinchikey, :smiles, :ficts, :ficus, :uuuuu, :sdf, :names, :hashisy, :image, :iupac_name]
				
				module_function
				
				def perform(identifier, representation, xml = false)
					if @@representation_type.include?(representation.to_sym)
						uri = "#{@@baseuri}/#{URI.escape(identifier, "[] ")}/#{representation.to_s}"
						
						begin
							return URI.parse(uri).read().strip().chomp.split(/\n/)
						rescue OpenURI::HTTPError
							return []
						end
						
					else
						raise Sylfy::Service::ParameterError, "Wrong representation type. Should be #{@@representation_type.join(', ')}"
					end
				end
				
            end
            
            module_function
            
			def getnames(inchi)
				result = []
				inchi = Rubabel[inchi, :inchi].to_s(:inchikey) if inchi =~ /^InChI\=1S\/[A-Za-z0-9]+(\/[cnpqbtmsih][A-Za-z0-9\-\+\(\)\,]+)+$/
				
				if inchi =~ /^[A-Z]{14}\-[A-Z]{10}(\-[A-N])?/
					result += REST.perform(inchi, :iupac_name)
					result += REST.perform(inchi, :names)
				end
				
				return result.uniq[0..21]
			end
             
		end
	end
end

#require './unisys.rb'
#
#a = Unisys::Service::RESTCactusNCI
#puts a.perform('Aspirin', :stdinchi)
#puts a.perform('(2E)-eicosenoyl-CoA', :stdinchi)
#puts a.perform('18-HEPE', :names)