# @author Natapol Pornputtapong <natapol@chalmers.se>
#
require 'open-uri'
require 'net/http'
require 'rexml/document'
require 'csv'

module Sylfy
  
		module CBioPortal
            
			module REST
				
				BASEURI = URI.parse('http://www.cbioportal.org')
				#BASEURI = URI.parse('http://www.cbioportal.org/public-portal/webservice.do?')
				HTTP = Net::HTTP.new(BASEURI.host, BASEURI.port)
				
				module_function
				# internal method supporting all ENSEMBLREST service
				#
				# @param get_path [String] service get_path
				#
				# @param option_type [Array] list of option type of service
				#
				# @param option [String] option of service please consult {http://beta.rest.ensembl.org/documentation/info/xref_id}
				#	
				# @return [array of Hash] results
				#
				def service(get_path, csv_opts = {})
					request = Net::HTTP::Get.new(get_path)
					response = HTTP.request(request)
					
					if response.code != "200"
						raise Sylfy::Service::ParameterError, "Invalid response: #{response.code}"
					else
                        return CSV.parse(response.body, {col_sep: "\t", skip_lines: /^#/, headers: true}.merge(csv_opts))
					end
				end
				
				# internal method supporting all ENSEMBLREST service
				#
				# @param get_path [String] service get_path
				#
				# @param option_type [Array] list of option type of service
				#
				# @param option [String] option of service please consult {http://beta.rest.ensembl.org/documentation/info/xref_id}
				#	
				# @return [array of Hash] results
				#
				def getTypesOfCancer()
					get_path = "/public-portal/webservice.do?cmd=getTypesOfCancer"
					return Sylfy::CBioPortal::REST.service(get_path)
				end
				
				# internal method supporting all ENSEMBLREST service
				#
				# @param get_path [String] service get_path
				#
				# @param option_type [Array] list of option type of service
				#
				# @param option [String] option of service please consult {http://beta.rest.ensembl.org/documentation/info/xref_id}
				#	
				# @return [array of Hash] results
				#
				def getCancerStudies()
					get_path = "/public-portal/webservice.do?cmd=getCancerStudies"
					return Sylfy::CBioPortal::REST.service(get_path)
				end
				
				# internal method supporting all ENSEMBLREST service
				#
				# @param cancer_study_id [String] service get_path
				#	
				# @return [array of Hash] results
				#
				def getGeneticProfiles(cancer_study_id)
					get_path = "/public-portal/webservice.do?cmd=getGeneticProfiles&cancer_study_id=#{cancer_study_id}"
					return Sylfy::CBioPortal::REST.service(get_path)
				end
				
				# internal method supporting all ENSEMBLREST service
				#
				# @param cancer_study_id [String] service get_path
				#	
				# @return [array of Hash] results
				#
				def getCaseLists(cancer_study_id)
					get_path = "/public-portal/webservice.do?cmd=getCaseLists&cancer_study_id=#{cancer_study_id}"
					return Sylfy::CBioPortal::REST.service(get_path)
				end
				
				
				# internal method supporting all ENSEMBLREST service
				#
				# @param case_set_id [String] case set ID (required)
				#
				# @param genetic_profile_id [String] one or more genetic profile IDs (required). Multiple genetic profile IDs must be separated by comma (,) characters, or URL encoded spaces, e.g. +
				#
				# @param gene_list [String] one or more genes, specified as HUGO Gene Symbols or Entrez Gene IDs (required). Multiple genes must be separated by comma (,) characters, or URL encoded spaces, e.g. +
				#
				# @return [array of Hash] results
				#
				def getProfileData(case_set_id, genetic_profile_id, gene_list)
					get_path = "/public-portal/webservice.do?cmd=getProfileData&case_set_id=#{case_set_id}&genetic_profile_id=#{genetic_profile_id}&gene_list=#{gene_list}"
					return Sylfy::CBioPortal::REST.service(get_path)
				end
				
				# internal method supporting all ENSEMBLREST service
				#
				# @param case_set_id [String] case set ID (required)
				#
				# @param genetic_profile_id [String] one or more genetic profile IDs (required). Multiple genetic profile IDs must be separated by comma (,) characters, or URL encoded spaces, e.g. +
				#
				# @param gene_list [String] one or more genes, specified as HUGO Gene Symbols or Entrez Gene IDs (required). Multiple genes must be separated by comma (,) characters, or URL encoded spaces, e.g. +
				#
				# @return [array of Hash] results
				#
				def getMutationData(case_set_id, genetic_profile_id, gene_list)
					get_path = "/public-portal/webservice.do?cmd=getMutationData&case_set_id=#{case_set_id}&genetic_profile_id=#{genetic_profile_id}&gene_list=#{gene_list}"
					return Sylfy::CBioPortal::REST.service(get_path)
				end
				
				# internal method supporting all ENSEMBLREST service
				#
				# @param case_set_id [String] case set ID (required)
				#
				# @return [array of Hash] results
				#
				def getClinicalData(case_set_id)
					get_path = "/public-portal/webservice.do?cmd=getClinicalData&case_set_id=#{case_set_id}"
					return Sylfy::CBioPortal::REST.service(get_path)
				end
				
				# internal method supporting all ENSEMBLREST service
				#
				# @param case_set_id [String] case set ID (required)
				#
				# @param genetic_profile_id [String] one or more genetic profile IDs (required). Multiple genetic profile IDs must be separated by comma (,) characters, or URL encoded spaces, e.g. +
				#
				# @param gene_list [String] one or more genes, specified as HUGO Gene Symbols or Entrez Gene IDs (required). Multiple genes must be separated by comma (,) characters, or URL encoded spaces, e.g. +
				#
				# @return [array of Hash] results
				#
				def getMutationData(case_set_id, genetic_profile_id, gene_list)
					get_path = "/public-portal/webservice.do?cmd=getMutationData&case_set_id=#{case_set_id}&genetic_profile_id=#{genetic_profile_id}&gene_list=#{gene_list}"
					return Sylfy::CBioPortal::REST.service(get_path)
				end
			end
			
		end
		
	
end


Sylfy::CBioPortal::REST.getMutationData("BRAF", "skcm_tcga_mutations").each {|e| p e; exit}
