# 
# UniSysDB library in Ruby
# Copyright (C) 2012
#
# @author Natapol Pornputtapong <natapol@chalmers.se>
#
require 'open-uri'
require 'net/http'
require 'json'

module Sylfy
	
	module Service
		
		
		module ChEMBL
			module REST
			
			
				@@baseuri = "https://www.ebi.ac.uk/chemblws"
				
				module_function
				
				def perform(input)
					
					uri = "#{@@baseuri}/#{input}"
					p uri
					begin
						return URI.parse(uri).read().strip()
					rescue OpenURI::HTTPError => e
						raise ServiceException, e
					end
				end
				
				private :perform
				# Method for setting inchi data member
				#
				# == Parameters:
				# id::
				#   LipidMap id to search
				# :id, :dataPrimarySource, :xrefs, :relations :inchi, :formula, :smiles, :inchiKey :names
				
				def compounds(id, domain = '', target = '', json = true)
					input = 'compounds'
					if [:stdinchikey, :smiles, :substructure].include?(domain.to_sym)
						input += "/#{domain}/#{id}"
						input += ".json" if json
					elsif domain.to_sym == :similaritiy 
						input += target.class == Integer ? "/#{domain}/#{id}/#{target}" : "/#{domain}/#{id}"
						input += ".json" if json
					elsif target.to_sym == :bioactivities
						input += "/#{id}/bioactivities"
						input += ".json" if json
					elsif target == :image || target =~ /^image?dimensions=(\d+)$/ || target =~ /^image=(\d+)$/
						if $1
							input += "/#{id}/image?dimensions=#{$1}"
						else
							input += "/#{id}/image?dimensions=300"
						end
					else
						input += "/#{id}"
						input += ".json" if json
					end
					
					return perform(input)
					
				end
				
				def targets(id, domain = '', bioactiv = false, json = true)
					input = 'targets'
					if id == ''
						
					elsif [:uniprot, :refseq].include?(domain.to_sym)
						input += "/#{domain}/#{id}"
						
					elsif bioactiv
						input += "/#{id}/bioactivities"
					else
						input += "/#{id}"
					end
					
					input += ".json" if json
					return perform(input)
					
				end
				
				def assays(id, bioactiv = false, json = true)
					input = "assays/#{id}"
					input += "/bioactivities" if bioactiv
					input += ".json" if json
					return perform(input)
					
				end
			end
		end
		
	end
	
end

#require 'json'
#require './unisys.rb'
#
#p JSON.load(Unisys::Service::RESTChEMBL.compounds('CHEMBL1'))
#p JSON.load(Unisys::Service::RESTChEMBL.targets('CHEMBL2477'))
#p JSON.load(Unisys::Service::RESTChEMBL.assays('CHEMBL1217643'))