# 
# Systems biology library for Ruby (Sylfy)
#
# author: Natapol Pornputtapong <natapol@chalmers.se>
#
# Documentation: Natapol Pornputtapong (RDoc'd and embellished by William Webber)
#

# raise "Please, use ruby 1.9.0 or later." if RUBY_VERSION < "1.9.0"


require 'sylfy/service/soapwsdl'

module Sylfy
    # A module supports as interfaces for web services
    module Service
        module ChEBI
            class SOAP < Sylfy::Service::SOAPWSDL
                def initialize(options = {})
                    super('http://www.ebi.ac.uk/webservices/chebi/2.0/webservice?wsdl', options)
                end
                
            end
            
            module_function
			
			# Search ChEBI database using InChI
			#
			# == Parameters:
			# inchi::
			#   inchi string to search
			def idbyinchi(inchi)
                output = Sylfy::Service::ChEBI::SOAP.new.get_structure_search(structure: Rubabel[inchi, :inchi].to_s(:can), type: 'SMILES', structureSearchCategory: 'IDENTITY')
                return output != nil ? ["urn:miriam:chebi:#{output[:list_element][:chebi_id]}"] : []
			end
			
        end
        
        module Miriam
            class SOAP < Sylfy::Service::SOAPWSDL
                def initialize(options = {})
                    super('http://www.ebi.ac.uk/miriamws/main/MiriamWebServices?wsdl', options)
                end
            end
        end
    end
end


#p Sylfy::Service::PubChem::REST.compound('QIQXTHQIDYTFRH-UHFFFAOYSA-N', :inchikey, 'xrefs/SBURL')["InformationList"]["Information"][0]["SBURL"]
#p Sylfy::Service::ChEBI::SOAP.new().get_complete_entity(chebiId: "CHEBI:15377")
#p Sylfy::Service::Miriam::SOAP.new().get_official_data_type_uri("nickname" => "ChEBI")
#p Sylfy::Service::Miriam::SOAP.new().convert_url(string: "urn:miriam:chebi:CHEBI:28842")
#p Sylfy::Service::Miriam::SOAP.new().get_official_data_type_uri(nickname: "chebi")
#p Sylfy::Service::Miriam::SOAP.new().get_miriam_uri(uri: "http://www.ebi.ac.uk/chebi/#CHEBI:17891")