# @author Natapol Pornputtapong <natapol@chalmers.se>
#
require 'open-uri'
require 'net/http'
require 'rexml/document'
require 'json'
require 'yaml'

require 'sylfy/service/ensemblrest/xrefs.rb'
require 'sylfy/service/ensemblrest/archive.rb'
require 'sylfy/service/ensemblrest/comparative.rb'

module Sylfy
	
	module Service
		
		module ENSEMBL
			module REST
			
				BASEURI = URI.parse("http://beta.rest.ensembl.org")
				HTTP = Net::HTTP.new(BASEURI.host, BASEURI.port)
				
				module_function
				
				# internal method supporting all ENSEMBLREST service
				#
				# @param get_path [String] service get_path
				#
				# @param option_type [Array] list of option type of service
				#
				# @param option [String] option of service please consult {http://beta.rest.ensembl.org/documentation/info/xref_id}
				#	
				# @return [array of Hash] results
				#
				def service(get_path, option_type = [], option={})
					
					optionList = []
						
					option.each do |k, v|
						optionList.push("#{k}=#{v}") if option_type.include?(k.to_sym)
					end
					
					
					get_path += "?#{optionList.join(";")}" if !optionList.empty?
					
					request = Net::HTTP::Get.new(get_path, {'Content-Type' => option.has_key?('content-type') ? option['content-type'] : 'application/json'})
					response = HTTP.request(request)
					
					if response.code != "200"
						#raise ParameterError, "Invalid response: #{response.code}"
					else
						if option.has_key?('content-type') && option['content-type'] != 'application/json'
							return response.body
						else
							return JSON.parse(response.body)
						end
					end
					
				end
			end
		end
		
	end
end