# @author Natapol Pornputtapong <natapol@chalmers.se>
#
require 'json'

module Sylfy
	
	module Service
	
		module ENSEMBL
			module REST
			
				module Xrefs
					
					SERVICENAME = 'xrefs'
					
					module_function
					
					# Perform lookups of Ensembl Identifiers and retrieve their external cross references
					# in other databases
					#
					# @param id [String] ENSEMBL id
					#
					# @param option [String] option of service please consult {http://beta.rest.ensembl.org/documentation/info/xref_id}
					#	
					# @return [array of Hash] results
					#
					def id(id, option = {})
						option_type = [:all_levels, :callback, :db_type, :external_db, :object_type, :species]
						get_path = "/#{SERVICENAME}/id/#{id}"
						return Sylfy::Service::ENSEMBLREST::service(get_path, option_type, option)
					end
					
					# Performs a lookup based upon the primary accession or display label of an external
					# reference and returning the information we hold about the entry
					#
					# @param species [String] Registry name/aliases used to restrict searches by
					#
					# @param name [String] The external name/primary accession to search for
					#
					# @param option [String] option of service please consult {http://beta.rest.ensembl.org/documentation/info/xref_id}
					#	
					# @return [array of Hash] results
					#
					def name(species, name, option = {})
						option_type = [:callback, :db_type, :external_db]
						get_path = "/#{SERVICENAME}/name/#{species}/#{name}"
						return Sylfy::Service::ENSEMBLREST::service(get_path, option_type, option)
					end
					
					# Looks up an external symbol and returns all Ensembl objects linked to it.
					# This can be a display name for a gene/transcript/translation, a synonym or an
					# externally linked reference. If a Gene's transcript is linked to the supplied
					# symbol the service will return both Gene and Transcript (it supports transient
					# links).
					#
					# @param species [String] Registry name/aliases used to restrict searches by
					#
					# @param symbol [String] The external name/symbol to search for
					#
					# @param option [String] option of service please consult {http://beta.rest.ensembl.org/documentation/info/xref_id}
					#	
					# @return [array of Hash] results
					#
					def symbol(species, symbol, option = {})
						option_type = [:callback, :db_type, :external_db, :object_type]
						get_path = "/#{SERVICENAME}/symbol/#{species}/#{symbol}"
						return Sylfy::Service::ENSEMBLREST::service(get_path, option_type, option)
					end
				end
			end
        end
    end
end
