# @author Natapol Pornputtapong <natapol@chalmers.se>
#

module Sylfy
	
	module Service
	
		module KEGGREST
			
			module_function
			# interface to KEGG conv service
			#
			# @param targetdb [Symbol] please consult {http://www.kegg.jp/kegg/rest/keggapi.html}
			#
			# @param sourcedb [String] can be database name or specific ID(s) joined with '+'
			#	
			# @return [Hash] results with sourcedb ID as keys
			# 
			def conv(targetdb, *sourcedb)
				
				begin
					doc = URI.parse("#{Sylfy::Service::KEGGREST::BASEURI}/conv/#{targetdb.to_s}/#{sourcedb.join("+")}").read().strip()
					result = {}
					
					doc.split(/\n/).each do |line|
						dat = line.chomp.split(/\t/)
						result[dat[0]] = [] if !result.has_key?(dat[0])
						result[dat[0]].push(dat[1])
					end
					
					return result
					
				rescue OpenURI::HTTPError
					raise Sylfy::Service::DataNotFound, "Query not found."
				end
				
			end
			
			def ncbi2ko(*sourcedb)
				begin
					ids = sourcedb.flatten.map { |x| x =~ /^\d+$/ ? "ncbi-geneid:" + x : x}
					hash1 = conv(:genes, ids)
					list1 = []
					hash1.each_value do |v|
						list1 += v
					end
					hash2 = link(:ko, list1)
					results = {}
					hash1.each_pair do |k, vs|
						results[k] ||= []
						vs.each {|v| results[k] = results[k] | hash2[v] if hash2.has_key?(v)}
					end
					
					results.delete_if {|key, value| value.empty? }
					
					return results
				rescue Sylfy::Service::DataNotFound
					return {}
				end
			end
			
			def ncbi2ko_org(org)
				
				results = {}
				
				begin
					hash1 = conv(org.to_s, 'ncbi-geneid')
					hash2 = link(:ko, org.to_s)
					hash1.each_pair do |k, vs|
						results[k] ||= []
						vs.each {|v| results[k] = results[k] | hash2[v] if hash2.has_key?(v)}
					end
					results.delete_if {|key, value| value.empty? }
					return results
				rescue
					return results
				end
				
				
			end
		
		end
		
	end
	
end
