# @author Natapol Pornputtapong <natapol@chalmers.se>
#

module Sylfy
	
	module Service
	
		module KEGGREST
			
			module_function
			# interface to KEGG find service
			#
			# @param database [Symbol] database name can be :pathway, :module, :disease, :drug, :environ, :ko, :genome, org, :compound, :glycan, :reaction, :rpair, :rclass, :enzyme, :genes, :ligand
			#
			# @param query [String] Query text keyword can be join with + sign or in double qoutw
			#   
			# @param option [Symbol] can be :formula, :exact_mass, :mol_weight with database = :compound, :drug
			#	
			# @return [Hash] results with sourcedb ID as keys
			#
			def find(database, query, option=nil)
				
				result_tag = {
					:pathway => [:NAME],
					:module => [:NAME],
					:disease => [:NAME],
					:drug => [:NAME],
					:environ => [:NAME],
					:ko => [:NAME, :DEFINITION],
					:genome => [:NAME, :DEFINITION],
					:compound => [:NAME],
					:glycan => [:NAME, :COMPOSITION, :CLASS],
					:reaction => [:NAME, :DEFINITION],
					:rpair => [:NAME],
					:rclass => [:NAME, :DEFINITION],
					:enzyme => [:NAME]
				}
				result_tag.default = [:NAME, :DEFINITION, :ORTHOLOGY]
				result = {}
				begin
					if ['compound', 'drug'].include?(database.to_s) && option != nil &&
						['formula', 'exact_mass' 'mol_weight'].include?(option.to_s)
						URI.parse("#{Sylfy::Service::KEGGREST::BASEURI}/find/#{database}/#{URI.escape(query)}/#{option}").read.strip.split(/\n/).each do |line|
							dat = line.chomp.split(/\t/)
							result[dat[0]] = { option.to_sym => dat[1]}
						end
					else
						URI.parse("#{Sylfy::Service::KEGGREST::BASEURI}/find/#{database}/#{URI.escape(query)}").read.strip.split(/\n/).each do |line|
							dat = line.chomp.split(/\t/)
							result[dat[0]] = {}
							info = dat[1].split('; ')
							info.each_index {|ind| result[dat[0]][result_tag[database.to_sym][ind]] = info[ind]}
						end
					end
					
					return result
				rescue OpenURI::HTTPError
					raise Unisys::ServiceException, "Query not found."
				end
				
			end
		
		end
		
	end
	
end

