# @author Natapol Pornputtapong <natapol@chalmers.se>
#

module Sylfy
	
	module Service
	
		module KEGGREST
			
			module_function
			# interface to KEGG get service
			#
			# @param dbentries [Symbol, String] can be :pathway, :brite, :module, :disease, :drug, :environ, :ko, :genome, :org, :compound, :glycan, :reaction, :rpair, :rclass, :enzyme
			#
			# @param option [Symbol] can be aaseq, :ntseq, :mol, :kcf, :image, :kgml
			#	
			# @return [Array] An array of Bio::KEGG object or string if option = :mol, :kcf, :kgml or Bio::FastaFormat if option =  :aaseq, :ntseq
			#
			def get(dbentries, option = nil)
				
				if option != nil && ['aaseq', 'ntseq', 'mol', 'kcf', 'kgml'].include?(option.to_s)
					text = "#{dbentries}/#{option}"
				else
					text = "#{dbentries}"
				end
				
				begin
					doc = URI.parse("#{Sylfy::Service::KEGGREST::BASEURI}/get/#{text}").read
					
				rescue OpenURI::HTTPError
					raise Sylfy::Service::DataNotFound, "Query not found."
				end
				
				if ['aaseq', 'ntseq'].include?(option.to_s)
					return Bio::FastaFormat.new(doc)
				elsif option.to_s == 'mol'
					return doc
				elsif option.to_s == 'kcf'
					return doc
				elsif option.to_s == 'kgml'
					return Bio::KEGG::KGML.new(doc.strip)
				else
					result = []
				
					doc.strip.split('///').each do |entry|
						case entry
						when /^ENTRY\s+[A-Z0-9]+\s+Glycan/
							result.push(Bio::KEGG::GLYCAN.new(entry))
						when /^ENTRY\s+[A-Z0-9]+\s+Compound/
							result.push(Bio::KEGG::COMPOUND.new(entry))
						when /^ENTRY\s+[A-Z0-9]+\s+CDS/
							result.push(Bio::KEGG::GENES.new(entry))
						when /^ENTRY\s+[A-Z0-9]+\s+Drug/
							result.push(Bio::KEGG::DRUG.new(entry))
						when /^ENTRY\s+[A-Z0-9]+\s+Reaction/
							result.push(Bio::KEGG::REACTION.new(entry))
						when /^ENTRY\s+[A-Z0-9]+\s+RPair/
							result.push(entry)
						when /^ENTRY\s+[A-Z0-9]+\s+Genome/
							result.push(Bio::KEGG::GENOME.new(entry))
						when /^ENTRY\s+[A-Z0-9]+\s+Enzyme/
							result.push(Bio::KEGG::ENZYME.new(entry))
						when /^ENTRY\s+[A-Z0-9]+\s+Module/
							result.push(Bio::KEGG::MODULE.new(entry))
						when /^ENTRY\s+[A-Z0-9]+\s+Pathway/
							result.push(Bio::KEGG::PATH.new(entry))
						else
							result.push(entry)
						end
					end
				end
				
				return result
				
			end
		
		end
		
	end
	
end

