# @author Natapol Pornputtapong <natapol@chalmers.se>
#

module Sylfy
	
	module Service
		
		module KEGGREST
			
			module_function
			# interface to KEGG list service
			#
			# @param database [Symbol] :pathway, :module, :disease, :drug, :environ, :ko, :genome, org, :compound, :glycan, :reaction, :rpair, :rclass, :enzyme, :genes, :ligand
			#
			# @param org [Symbol] organism code can be :hsa, :eco, etc {http://www.kegg.jp/kegg/rest/keggapi.html}
			#
			# @return [Hash] results with ID and description
			#
			def list(database, org = nil)
				
				begin
					doc = URI.parse("#{Sylfy::Service::KEGGREST::BASEURI}/list/#{database.to_s}#{org ? "/#{org}" : ""}").read.strip
					result = {}
					doc.split(/\n/).each do |line|
						dat = line.chomp.split(/\t/, 2)
						result[dat[0]] = dat[1]
					end
					
					return result
				rescue OpenURI::HTTPError
					raise Sylfy::Service::DataNotFound, "Data not found."
				end
				
			end
			
		end
	end
	
end

