# 
# UniSysDB library in Ruby
# Copyright (C) 2012
#
# @author Natapol Pornputtapong <natapol@chalmers.se>
#
require 'open-uri'
require 'net/http'
require 'json'

require 'sylfy/service/pubchemrest'

module Sylfy
	
	module Service
		
		module PubChem
			
			module_function
			
			# Method for setting inchi data member
			#
			# == Parameters:
			# id::
			#   LipidMap id to search
			# :id, :dataPrimarySource, :xrefs, :relations :inchi, :formula, :smiles, :inchiKey :names
			def idbyinchi(inchi)
				result = []
				if inchi =~ /^InChI\=1S\/[A-Za-z0-9]+(\/[cnpqbtmsih][A-Za-z0-9\-\+\(\)\,]+)+$/
					REST::compound(Rubabel[inchi, :inchi].write(:inchikey), 'inchikey', "cids")["IdentifierList"]["CID"].each {|e| result.push("urn:miriam:pubchem.compound:#{e}")}
				elsif inchi =~ /^[A-Z]{14}\-[A-Z]{10}(\-[A-N])?/
					REST::compound(inchi, 'inchikey', "cids")["IdentifierList"]["CID"].each {|e| result.push("urn:miriam:pubchem.compound:#{e}")}
				end
				
				return result
			end
			
			# Method for setting inchi data member
			#
			# == Parameters:
			# id::
			#   LipidMap id to search
			# :id, :dataPrimarySource, :xrefs, :relations :inchi, :formula, :smiles, :inchiKey :names
			def getnames(inchi)
				result = []
				inchi = Rubabel[inchi, :inchi].to_s(:inchikey) if inchi =~ /^InChI\=1S\/[A-Za-z0-9]+(\/[cnpqbtmsih][A-Za-z0-9\-\+\(\)\,]+)+$/
				
				if inchi =~ /^[A-Z]{14}\-[A-Z]{10}(\-[A-N])?/
					Sylfy::Service::PubChem::REST.compound(inchi, :inchikey, "property/IUPACName")["PropertyTable"]["Properties"].each {|e| result.push(e["IUPACName"])}
					Sylfy::Service::PubChem::REST.compound(inchi, :inchikey, :synonyms)["InformationList"]["Information"].each {|e| result += e["Synonym"]}
				end
				
				return result.uniq[0..21]
			end
		end
	end
end



#require 'json'
#require './unisys.rb'

#http://pubchem.ncbi.nlm.nih.gov/rest/pug/assay/aid/1000/sids/XML?sids_type=inactive

#p JSON.load(Unisys::Service::RESTPubChem.assay("1000", :aid, :sids, :JSON ,{:sids_type => :inactive}))
#p JSON.load(Unisys::Service::RESTPubChem.substance("747285", "sourceid/DTP.NCI"))
#p JSON.load(Unisys::Service::RESTPubChem.compound("2244", :cid, "property/MolecularFormula,CanonicalSMILES,InChI,InChIKey,IUPACName", "JSON"))
#p Unisys::Service::RESTPubChem.getSmallMolecule(2244, :cid)
#p Unisys::Service::RESTPubChem.idconv('D00001')
#p JSON.load(Unisys::Service::RESTPubChem.compound("LMFA08040013", 'xref/RegistryID', "property/MolecularFormula,CanonicalSMILES,InChI,InChIKey,IUPACName", "JSON"))
