# 
# Sylfy library in Ruby
# Copyright (C) 2012
#
# @author Natapol Pornputtapong <natapol@chalmers.se>
#
require 'open-uri'
require 'net/http'
require 'json'


module Sylfy
	
    module Service
    
        module UniChem
        
            module REST
            
                @@baseuri = "https://www.ebi.ac.uk/unichem/rest"
                NAMEURIMAPPING = {
					chembl: "chembl.compound",
					drugbank: "drugbank",
					pdb: "pdb",
					iuphar: "iuphar.ligand",
					kegg_ligand: "kegg.compound",
					chebi: "chebi",
					#emolecules: "",
					#ibm: "",
					#atlas: "",
					#patents: "",
					#fdasrs: "",
					#surechem: "",
					pharmgkb: "pharmgkb.drug",
					#pubchem_tpharma: "",
					hmdb: "hmdb",
					pubchem: "pubchem.compound"
					#mcule: ""
				}
                
                module_function
                
                def perform(input)
                    
                    uri = "#{@@baseuri}/#{input}"
                    
                    begin
                        return JSON.parse(open(uri, "content_type" => 'application/json').read().strip())
                    rescue OpenURI::HTTPError
                        return JSON.parse("{}")
                    end
                end
                
                def src_compound_id(src_cpd_id, src_id = 1, to_src_id = nil, all = false)
                    input = "src_compound_id#{all == true ? "_all" : ''}/#{src_cpd_id}/#{src_id}#{to_src_id != nil ? "/#{to_src_id}" : ""}"
                    return perform(input)
                end
                
				def src_compound_id_all(src_cpd_id, src_id = 1, to_src_id = nil)
					return src_compound_id(src_cpd_id, src_id, to_src_id, true)
				end
                
				def mapping(src_id, to_src_id)
					input = "mapping/#{src_id}/#{to_src_id}"
					return perform(input)
				end
				
				def inchikey(key, all = false)
					input = "inchikey#{all == true ? "_all" : ''}/#{key}"
					return perform(input)
				end
				
				def inchikey_all(key)
					return inchikey(key, true)
				end
				
				def structure(src_cpd_id, src_id)
					input = "structure/#{src_cpd_id}/#{src_id}"
					return perform(input)
				end
				
				def verbose_inchikey(key)
					input = "verbose_inchikey/#{key}"
					return perform(input)
				end
            end
            
            module_function
                
			def idbyinchi(inchi)
				result = []
				inchi = Rubabel[inchi, :inchi].to_s(:inchikey) if inchi =~ Sylfy::Pattern::INCHI
				
				if inchi =~ Sylfy::Pattern::INCHIKEY
					REST.verbose_inchikey(inchi).each do |entry|
						if REST::NAMEURIMAPPING.has_key?(entry["name"].to_sym)
							entry["src_compound_id"].each {|e| result.push("urn:miriam:#{REST::NAMEURIMAPPING[entry["name"].to_sym]}:#{e}")}
						end
					end
				end
				
				return result
			end
        end
    end
end