require 'open-uri'

# additional module of bio ruby library
module Bio
# Class Bio::KEGG
class KEGG
    # Class Bio::KEGG::KGML
    class KGML
        # convert KGML to mjson (map json)
        def to_mjson()
            mjson = {:identifier => :id}
            
            # <pathway> attributes
            mjson[:name] = @name
            mjson[:org] = @org
            mjson[:number] = @number
            mjson[:title] = @title
            mjson[:link] = @link
            
            
            mjson[:image] = {}
            mjson[:image][:source] = @image
            
            begin
                dim = open(@image).read()[0x10..0x18].unpack('NN')
                mjson[:image][:width] = dim[0]
                mjson[:image][:height] = dim[1]
            rescue
                
            end
            
            
            # <entry> attributes
            @entries.each do |entry|
                entryinfo = {:id => entry.id, :entryname => entry.name, :link => entry.link, :entrytype => entry.type, :keggid => entry.names}
              
                entryinfo[:reaction] = entry.reaction if entry.reaction
                
                # <graphics> attributes
                graphicinfo = {}
                if entry.graphics.length == 1
                    graphicinfo[:fill] = entry.graphics[0].bgcolor if entry.graphics[0].bgcolor
                    graphicinfo[:stroke] = entry.graphics[0].fgcolor if entry.graphics[0].fgcolor
                    graphicinfo[:name] = entry.graphics[0].name if entry.graphics[0].name
            
                    case entry.graphics[0].type
                    when 'rectangle'
                        graphicinfo[:type] = :rect
                        graphicinfo[:width] = entry.graphics[0].width
                        graphicinfo[:height] = entry.graphics[0].height
                        graphicinfo[:x] = entry.graphics[0].x
                        graphicinfo[:y] = entry.graphics[0].y
                    when 'roundrectangle'
                        graphicinfo[:type] = :rect
                        graphicinfo[:width] = entry.graphics[0].width
                        graphicinfo[:height] = entry.graphics[0].height
                        graphicinfo[:x] = entry.graphics[0].x
                        graphicinfo[:y] = entry.graphics[0].y
                        graphicinfo[:r] = 10
                    when 'circle'
                        graphicinfo[:type] = :circle
                        graphicinfo[:r] = entry.graphics[0].width/2
                        graphicinfo[:cx] = entry.graphics[0].x
                        graphicinfo[:cy] = entry.graphics[0].y
                    when 'line'
                        graphicinfo[:type] = :path
                        paths = []
                        entry.graphics[0].coords.each {|coord| paths.push(coord.join(" "))}
                        graphicinfo[:fill] = entry.graphics[0].bgcolor if entry.graphics[0].bgcolor
                        graphicinfo[:stroke] = entry.graphics[0].fgcolor if entry.graphics[0].fgcolor
                        graphicinfo[:name] = entry.graphics[0].name if entry.graphics[0].name
                        graphicinfo[:d] ||= ""
                        graphicinfo[:d] += "M #{paths.join("L")}" if !paths.empty?()
                        graphicinfo[:"stroke-width"] = 3
                        graphicinfo[:"stroke-linecap"] = "square"
                    end
                    
                else
                    entry.graphics.each do |graph|
                        if graph.type == 'line'
                            paths = []
                            graph.coords.each {|coord| paths.push(coord.join(" "))}
                            graphicinfo[:type] = :path
                            graphicinfo[:fill] = graph.bgcolor if graph.bgcolor
                            graphicinfo[:stroke] = graph.fgcolor if graph.fgcolor
                            graphicinfo[:name] = graph.name if graph.name
                            graphicinfo[:d] ||= ""
                            graphicinfo[:d] += "M #{paths.join("L")}" if !paths.empty?()
                            graphicinfo[:"stroke-width"] = 3
                            graphicinfo[:"stroke-linecap"] = "square"
                        else
                            raise "Graphic parsinf error to #{id}:#{entry.id}"
                        end
                    end
                end
                
                entryinfo[:graphic] = graphicinfo
                mjson[graphicinfo[:type]] ||= []
                mjson[graphicinfo[:type]].push(entryinfo)
            
                # <component> attributes
                
            end
            return mjson
        end
    end
end
end
